@@ -221,16 +221,22 @@ def unzip_file(filename, dir=None, verbose=False):
 class Repository(object):
 
     svn_get='checkout'
+    useEasyInstall = False
     easy_install_path = ["easy_install"]
-    easy_install_flag = '-q'
+    easy_install_flags = ['-q']
     pip_path = ["pip"]
+    pip_flags = ['-q']
     python = "python"
     svn = "svn"
     dev = []
 
     def __init__(self, name, **kwds):
         class _TEMP_(object):
-            def __init__(self, root=None, trunk=None, stable=None, release=None, tag=None, pyname=None, pypi=None, dev=False, username=None, install=True, rev=None, local=None, platform=None, version=None, branch=None, exit=True):
+            def __init__( self, root=None, trunk=None, stable=None, 
+                          release=None, tag=None, pyname=None, pypi=None, 
+                          dev=False, username=None, install=True, rev=None, 
+                          local=None, platform=None, version=None, 
+                          branch=None, exit=True ):
                 import inspect
                 args, varargs, varkw, defaults = inspect.getargspec(self.__init__)
                 for i in range(len(args)-1):
@@ -471,7 +477,7 @@ class Repository(object):
         if not self.version is None and not eval(self.version):
             return
         if self.pkgdir is None and self.local is None:
-            self.easy_install(install, preinstall, dir, offline)
+            self.install_package(install, preinstall, dir, offline)
             return
         if self.local:
             install = True
@@ -499,10 +505,14 @@ class Repository(object):
             try:
                 self.run([self.svn]+self.svn_username+[Repository.svn_get,'-q',self.pkgdir+self.rev, dir])
             except OSError:
-                err = sys.exc_info()[1] # BUG?
+                err,tb = sys.exc_info()[1:3] # BUG?
                 print("")
                 print("Error checkout software %s with subversion at %s" % (self.name,self.pkgdir+self.rev))
                 print(str(err))
+                print("Traceback:")
+                import traceback
+                traceback.print_stack(f=tb.tb_frame.f_back,file=sys.stdout)
+                traceback.print_tb(tb, file=sys.stdout)
                 if self.config.exit:
                     print("Aborting installer!")
                     sys.exit(1)
@@ -518,11 +528,15 @@ class Repository(object):
                 else:
                     self.run([self.python, 'setup.py', 'install'], dir=dir)
             except OSError:
-                err = sys.exc_info()[1] # BUG?
+                err,tb = sys.exc_info()[1:3] # BUG?
                 print("")
                 print("Error installing software %s from source using the setup.py file." % self.name)
                 print("This is probably due to a syntax or configuration error in this package.")
                 print(str(err))
+                print("Traceback:")
+                import traceback
+                traceback.print_stack(f=tb.tb_frame.f_back,file=sys.stdout)
+                traceback.print_tb(tb, file=sys.stdout)
                 if self.config.exit:
                     print("Aborting installer!")
                     sys.exit(1)
@@ -542,7 +556,7 @@ class Repository(object):
 
     def perform_update(self, dir=None):
         if self.pkgdir is None:
-            self.easy_upgrade()
+            self.upgrade_packages()
             return
         print("-----------------------------------------------------------------")
         print("  Updating branch")
@@ -556,22 +570,42 @@ class Repository(object):
         else:
             self.run([self.python, 'setup.py', 'install'], dir=dir)
 
+    def install_package(self, install, preinstall, dir, offline):
+        if Repository.useEasyInstall:
+            return self.easy_install(install, preinstall, dir, offline)
+        else:
+            try:
+                return self.pip_install(install, preinstall, dir, offline)
+            except:
+                print("ERROR installing with PIP; falling back on easy_install")
+            return self.easy_install(install, preinstall, dir, offline)
+
     def easy_install(self, install, preinstall, dir, offline):
-        #return self.pip_install(install, preinstall, dir, offline)
         try:
             if install:
                 if offline:
                     self.run([self.python, 'setup.py', 'install'], dir=dir)
                 else:
-                    self.run(self.easy_install_path + [Repository.easy_install_flag, self.pypi], dir=os.path.dirname(dir))
+                    self.run( self.easy_install_path 
+                              + Repository.easy_install_flags + [self.pypi], 
+                              dir=os.path.dirname(dir))
             elif preinstall:
                 if not os.path.exists(dir):
-                    self.run(self.easy_install_path + [Repository.easy_install_flag, '--exclude-scripts', '--always-copy', '--editable', '--build-directory', '.', self.pypi], dir=os.path.dirname(dir))
+                    self.run(self.easy_install_path 
+                             + Repository.easy_install_flags 
+                             + [ '--exclude-scripts', '--always-copy', 
+                                 '--editable', '--build-directory', '.', 
+                                 self.pypi], 
+                             dir=os.path.dirname(dir))
         except OSError:
-            err = sys.exc_info()[1] # BUG?
+            err,tb = sys.exc_info()[1:3] # BUG?
             print("")
             print("Error installing package %s with easy_install" % self.name)
             print(str(err))
+            print("Traceback:")
+            import traceback
+            traceback.print_stack(f=tb.tb_frame.f_back,file=sys.stdout)
+            traceback.print_tb(tb, file=sys.stdout)
             if self.config.exit:
                 print("Aborting installer!")
                 sys.exit(1)
@@ -583,29 +617,51 @@ class Repository(object):
                 if offline:
                     self.run([self.python, 'setup.py', 'install'], dir=dir)
                 else:
-                    self.run(self.pip_path + ['-v', self.pypi])
+                    self.run( self.pip_path + ['install'] 
+                              + Repository.pip_flags + [self.pypi],
+                              dir=os.path.dirname(dir))
             elif preinstall:
                 if not os.path.exists(dir):
-                    self.run(self.pip_path + ['-v', '--no-install', '--download', '.', self.pypi], dir=os.path.dirname(dir))
+                    self.run( self.pip_path + ['install'] 
+                              + Repository.pip_flags 
+                              + [ '--no-install', '--ignore-installed',
+                                  '--build', '.', self.pypi ],
+                              dir=os.path.dirname(dir))
         except OSError:
-            err = sys.exc_info()[1] # BUG?
+            err,tb = sys.exc_info()[1:3] # BUG?
             print("")
             print("Error installing package %s with pip" % self.name)
             print(str(err))
+            print("Traceback:")
+            import traceback
+            traceback.print_stack(f=tb.tb_frame.f_back,file=sys.stdout)
+            traceback.print_tb(tb, file=sys.stdout)
             if self.config.exit:
                 print("Aborting installer!")
                 sys.exit(1)
             print("Not aborting installer...")
 
+    def upgrade_packages(self):
+        if Repository.useEasyInstall:
+            return self.easy_upgrade()
+        else:
+            return self.pip_upgrade()
+
     def easy_upgrade(self):
-        self.run(self.easy_install_path + [Repository.easy_install_flag, '--upgrade', self.pypi])
+        self.run( self.easy_install_path + Repository.easy_install_flags 
+                  + ['--upgrade', self.pypi] )
+
+    def pip_upgrade(self):
+        self.run( self.pip_path + ['install'] + Repository.pip_flags 
+                  + ['--upgrade', self.pypi] )
 
     def run(self, cmd, dir=None):
         cwd=os.getcwd()
         if not dir is None:
             os.chdir(dir)
             cwd=dir
-        print("Running command '%s' in directory %s" % (" ".join(cmd), cwd))
+        print( "\nRunning command '%s'\n\tin directory %s" 
+               % (" ".join(cmd), cwd) )
         sys.stdout.flush()
         call_subprocess(cmd, filter_stdout=filter_python_develop, show_stdout=True)
         if not dir is None:
@@ -799,6 +855,14 @@ class Installer(object):
             dest='localize',
             default=False)
 
+        parser.add_option('--pypi-url',
+            help='Provide an alernative index_url to easy_install '
+                 '(e.g., to redirect the installer to a local PyPI mirror)',
+            action='store',
+            dest='pypi_url',
+            default=None)
+
+
         #
         # Change the virtualenv options
         #
@@ -832,7 +896,15 @@ class Installer(object):
         #
         global logger
         if options.verbose:
-            Repository.easy_install_flag = '-v'
+            # This works because the -v will appear after (and override)
+            # the default -q on the final command line.
+            if '-q' in Repository.easy_install_flags:
+                Repository.easy_install_flags.remove('-q')
+            Repository.easy_install_flags.append('-v')
+
+            if '-q' in Repository.pip_flags:
+                Repository.pip_flags.remove('-q')
+            Repository.pip_flags.append('-v')
         verbosity = options.verbose - options.quiet
         self.logger = Logger([(Logger.level_for_integer(2-verbosity), sys.stdout)])
         logger = self.logger
@@ -886,6 +958,15 @@ class Installer(object):
         #
         if options.preinstall:
             Repository.svn_get='export'
+        #
+        # If the user requested an alternative PyPI index, add it to the
+        # easy_install command line.
+        #
+        if options.pypi_url:
+            Repository.easy_install_flags.append("--index-url")
+            Repository.easy_install_flags.append(options.pypi_url)
+            Repository.pip_flags.append("--index-url")
+            Repository.pip_flags.append(options.pypi_url)
 
     def get_homedir(self, options, args):
         #
